@@ -12,14 +12,13 @@ restricted by GSA ADP Schedule Contract with IBM Corp.
 package config
 
 import (
-	"os"
 	"net"
 	"runtime"
 	"strconv"
 
 	"github.com/golang/glog"
 
-	"github.com/stolostron/management-ingress/pkg/ingress"
+	"github.com/open-cluster-management/management-ingress/pkg/ingress"
 )
 
 const (
@@ -54,8 +53,7 @@ const (
 
 	// Enabled ciphers list to enabled. The ciphers are specified in the format understood by the OpenSSL library
 	// http://nginx.org/en/docs/http/ngx_http_ssl_module.html#ssl_ciphers
-	sslCiphers = "ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256"
-        sslCiphersFIPS = "ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256"
+	sslCiphersFIPS = "ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256"
 
 	// SSL enabled protocols to use
 	// http://nginx.org/en/docs/http/ngx_http_ssl_module.html#ssl_protocols
@@ -469,25 +467,25 @@ func NewDefault() Configuration {
 	}
 
 	cfg := Configuration{
-		AllowBackendServerHeader:   false,
-		AccessLogPath:              "/var/log/nginx/access.log",
-		ErrorLogPath:               "/var/log/nginx/error.log",
-		BrotliLevel:                4,
-		BrotliTypes:                brotliTypes,
-		ClientHeaderBufferSize:     "1k",
-		ClientHeaderTimeout:        60,
-		ClientBodyBufferSize:       "8k",
-		ClientBodyTimeout:          60,
-		EnableDynamicTLSRecords:    true,
-		EnableUnderscoresInHeaders: false,
-		ErrorLogLevel:              errorLevel,
-		DisableAccessLog:           true,
-		ForwardedForHeader:         "X-Forwarded-For",
-		ComputeFullForwardedFor:    false,
-		HTTP2MaxFieldSize:          "4k",
-		HTTP2MaxHeaderSize:         "16k",
-		HTTPRedirectCode:           308,
-		HSTS:                       true,
+		AllowBackendServerHeader:     false,
+		AccessLogPath:                "/var/log/nginx/access.log",
+		ErrorLogPath:                 "/var/log/nginx/error.log",
+		BrotliLevel:                  4,
+		BrotliTypes:                  brotliTypes,
+		ClientHeaderBufferSize:       "1k",
+		ClientHeaderTimeout:          60,
+		ClientBodyBufferSize:         "8k",
+		ClientBodyTimeout:            60,
+		EnableDynamicTLSRecords:      true,
+		EnableUnderscoresInHeaders:   false,
+		ErrorLogLevel:                errorLevel,
+		DisableAccessLog:             true,
+		ForwardedForHeader:           "X-Forwarded-For",
+		ComputeFullForwardedFor:      false,
+		HTTP2MaxFieldSize:            "4k",
+		HTTP2MaxHeaderSize:           "16k",
+		HTTPRedirectCode:             308,
+		HSTS:                         true,
 		HSTSIncludeSubdomains:        true,
 		HSTSMaxAge:                   hstsMaxAge,
 		HSTSPreload:                  false,
@@ -508,7 +506,7 @@ func NewDefault() Configuration {
 		ProxyStreamResponses:         1,
 		ShowServerTokens:             false,
 		SSLBufferSize:                sslBufferSize,
-		SSLCiphers:                   sslCiphers,
+		SSLCiphers:                   sslCiphersFIPS,
 		SSLECDHCurve:                 "auto",
 		SSLProtocols:                 sslProtocols,
 		SSLSessionCache:              true,
@@ -534,11 +532,6 @@ func NewDefault() Configuration {
 		ZipkinServiceName:            "nginx",
 	}
 
-	if os.Getenv("FIPS_ENABLED") == "true" {
-                cfg.SSLCiphers = sslCiphersFIPS
-	}
-
-
 	if glog.V(5) {
 		cfg.ErrorLogLevel = "debug"
 	}
