@@ -15,6 +15,7 @@
 
 
 #include "ngx_http_lua_common.h"
+#include "ngx_http_lua_ssl.h"
 #include "ngx_http_lua_api.h"
 
 
@@ -31,6 +32,14 @@
 
 #define NGX_HTTP_LUA_ESCAPE_HEADER_VALUE  8
 
+#define NGX_HTTP_LUA_CONTEXT_YIELDABLE (NGX_HTTP_LUA_CONTEXT_REWRITE         \
+                                        | NGX_HTTP_LUA_CONTEXT_ACCESS        \
+                                        | NGX_HTTP_LUA_CONTEXT_CONTENT       \
+                                        | NGX_HTTP_LUA_CONTEXT_TIMER         \
+                                        | NGX_HTTP_LUA_CONTEXT_SSL_CERT      \
+                                        | NGX_HTTP_LUA_CONTEXT_SSL_SESS_FETCH)
+
+
 /* key in Lua vm registry for all the "ngx.ctx" tables */
 #define ngx_http_lua_ctx_tables_key  "ngx_lua_ctx_tables"
 
@@ -45,6 +54,7 @@
      : (c) == NGX_HTTP_LUA_CONTEXT_BODY_FILTER ? "body_filter_by_lua*"       \
      : (c) == NGX_HTTP_LUA_CONTEXT_TIMER ? "ngx.timer"                       \
      : (c) == NGX_HTTP_LUA_CONTEXT_INIT_WORKER ? "init_worker_by_lua*"       \
+     : (c) == NGX_HTTP_LUA_CONTEXT_EXIT_WORKER ? "exit_worker_by_lua*"       \
      : (c) == NGX_HTTP_LUA_CONTEXT_BALANCER ? "balancer_by_lua*"             \
      : (c) == NGX_HTTP_LUA_CONTEXT_SSL_CERT ? "ssl_certificate_by_lua*"      \
      : (c) == NGX_HTTP_LUA_CONTEXT_SSL_SESS_STORE ?                          \
@@ -256,6 +266,8 @@ ngx_http_lua_init_ctx(ngx_http_request_t *r, ngx_http_lua_ctx_t *ctx)
     ngx_memzero(ctx, sizeof(ngx_http_lua_ctx_t));
     ctx->ctx_ref = LUA_NOREF;
     ctx->entry_co_ctx.co_ref = LUA_NOREF;
+    ctx->entry_co_ctx.next_zombie_child_thread =
+        &ctx->entry_co_ctx.zombie_child_threads;
     ctx->resume_handler = ngx_http_lua_wev_handler;
     ctx->request = r;
 }
@@ -388,6 +400,15 @@ ngx_http_lua_set_req(lua_State *L, ngx_http_request_t *r)
 }
 
 
+static ngx_inline void
+ngx_http_lua_attach_co_ctx_to_L(lua_State *L, ngx_http_lua_co_ctx_t *coctx)
+{
+#ifdef HAVE_LUA_EXDATA2
+    lua_setexdata2(L, (void *) coctx);
+#endif
+}
+
+
 #ifndef OPENRESTY_LUAJIT
 static ngx_inline void
 ngx_http_lua_get_globals_table(lua_State *L)
@@ -534,6 +555,134 @@ ngx_http_lua_check_unsafe_uri_bytes(ngx_http_request_t *r, u_char *str,
 }
 
 
+static ngx_inline void
+ngx_http_lua_free_thread(ngx_http_request_t *r, lua_State *L, int co_ref,
+    lua_State *co, ngx_http_lua_main_conf_t *lmcf)
+{
+#ifdef HAVE_LUA_RESETTHREAD
+    ngx_queue_t                 *q;
+    ngx_http_lua_thread_ref_t   *tref ;
+    ngx_http_lua_ctx_t          *ctx;
+
+    ngx_log_debug2(NGX_LOG_DEBUG_HTTP,
+                   r == NULL ? ngx_cycle->log : r->connection->log, 0,
+                   "lua freeing light thread %p (ref %d)", co, co_ref);
+
+    ctx = r != NULL ? ngx_http_get_module_ctx(r, ngx_http_lua_module) : NULL;
+    if (ctx != NULL
+        && L == ctx->entry_co_ctx.co
+        && L == lmcf->lua
+        && !ngx_queue_empty(&lmcf->free_lua_threads))
+    {
+        lua_resetthread(L, co);
+
+        q = ngx_queue_head(&lmcf->free_lua_threads);
+        tref = ngx_queue_data(q, ngx_http_lua_thread_ref_t, queue);
+
+        ngx_http_lua_assert(tref->ref == LUA_NOREF);
+        ngx_http_lua_assert(tref->co == NULL);
+
+        tref->ref = co_ref;
+        tref->co = co;
+
+        ngx_queue_remove(q);
+        ngx_queue_insert_head(&lmcf->cached_lua_threads, q);
+
+        ngx_log_debug2(NGX_LOG_DEBUG_HTTP,
+                       r != NULL ? r->connection->log : ngx_cycle->log, 0,
+                       "lua caching unused lua thread %p (ref %d)", co,
+                       co_ref);
+
+        return;
+    }
+#endif
+
+    ngx_log_debug2(NGX_LOG_DEBUG_HTTP,
+                   r != NULL ? r->connection->log : ngx_cycle->log, 0,
+                   "lua unref lua thread %p (ref %d)", co, co_ref);
+
+    lua_pushlightuserdata(L, ngx_http_lua_lightudata_mask(
+                          coroutines_key));
+    lua_rawget(L, LUA_REGISTRYINDEX);
+
+    luaL_unref(L, -1, co_ref);
+    lua_pop(L, 1);
+}
+
+
+static ngx_inline int
+ngx_http_lua_new_cached_thread(lua_State *L, lua_State **out_co,
+    ngx_http_lua_main_conf_t *lmcf, int set_globals)
+{
+    int                          co_ref;
+    lua_State                   *co;
+
+#ifdef HAVE_LUA_RESETTHREAD
+    ngx_queue_t                 *q;
+    ngx_http_lua_thread_ref_t   *tref;
+
+    if (L == lmcf->lua && !ngx_queue_empty(&lmcf->cached_lua_threads)) {
+        q = ngx_queue_head(&lmcf->cached_lua_threads);
+        tref = ngx_queue_data(q, ngx_http_lua_thread_ref_t, queue);
+
+        ngx_http_lua_assert(tref->ref != LUA_NOREF);
+        ngx_http_lua_assert(tref->co != NULL);
+
+        co = tref->co;
+        co_ref = tref->ref;
+
+        tref->co = NULL;
+        tref->ref = LUA_NOREF;
+
+        ngx_queue_remove(q);
+        ngx_queue_insert_head(&lmcf->free_lua_threads, q);
+
+        ngx_log_debug2(NGX_LOG_DEBUG_HTTP, ngx_cycle->log, 0,
+                       "lua reusing cached lua thread %p (ref %d)", co, co_ref);
+
+        lua_pushlightuserdata(L, ngx_http_lua_lightudata_mask(
+                              coroutines_key));
+        lua_rawget(L, LUA_REGISTRYINDEX);
+        lua_rawgeti(L, -1, co_ref);
+
+    } else {
+#else
+    {
+#endif
+        lua_pushlightuserdata(L, ngx_http_lua_lightudata_mask(
+                              coroutines_key));
+        lua_rawget(L, LUA_REGISTRYINDEX);
+        co = lua_newthread(L);
+        lua_pushvalue(L, -1);
+        co_ref = luaL_ref(L, -3);
+
+        ngx_log_debug2(NGX_LOG_DEBUG_HTTP, ngx_cycle->log, 0,
+                       "lua ref lua thread %p (ref %d)", co, co_ref);
+
+#ifndef OPENRESTY_LUAJIT
+        if (set_globals) {
+            lua_createtable(co, 0, 0);  /* the new globals table */
+
+            /* co stack: global_tb */
+
+            lua_createtable(co, 0, 1);  /* the metatable */
+            ngx_http_lua_get_globals_table(co);
+            lua_setfield(co, -2, "__index");
+            lua_setmetatable(co, -2);
+
+            /* co stack: global_tb */
+
+            ngx_http_lua_set_globals_table(co);
+        }
+#endif
+    }
+
+    *out_co = co;
+
+    return co_ref;
+}
+
+
 extern ngx_uint_t  ngx_http_lua_location_hash;
 extern ngx_uint_t  ngx_http_lua_content_length_hash;
 
